import jinja2

from .utils import get_experiment_variant


@jinja2.contextfunction
def experiment_variant(context, experiment):
    try:
        request = context['request']
    except KeyError:
        return jinja2.Undefined('request must be defined in Context')
    return get_experiment_variant(request, experiment, make_decision=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import random

from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Experiment(models.Model):
    name = models.CharField(
        max_length=50, unique=True,
        help_text='This is the name you\'ll use to reference this experiment '
        'in code and templates.')
    description = models.TextField(blank=True)
    active = models.BooleanField(default=False)
    variants = models.TextField(
        help_text='Provide a list of all variants you\'d like to test, one '
        'variant name per line')

    def __str__(self):
        return self.name

    def get_variants(self):
        try:
            return self._variant_cache
        except AttributeError:
            self._variant_cache = [v.strip() for v in
                                   self.variants.split('\n')]
        return self._variant_cache

    def choose_variant(self):
        variants = self.get_variants()
        return random.choice(variants)



from collections import defaultdict
from .utils import get_experiment_variant


class VariantTestMixin(object):
    experiments = []

    def __init__(self, *args, **kwargs):
        super(VariantTestMixin, self).__init__(*args, **kwargs)
        self.variants = defaultdict(lambda: None)

    def dispatch(self, request, *args, **kwargs):
        self.set_variants()
        return super(VariantTestMixin, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs['variants'] = self.variants
        return super(VariantTestMixin, self).get_context_data(**kwargs)

    def set_variants(self):
        for exp in self.experiments:
            variant = get_experiment_variant(self.request, exp)
            if variant:
                self.variants[exp] = variant



#!/usr/bin/env python

from setuptools import setup

setup(
    name='django-variant',
    version='0.1.1',
    description='Django variant testing framework',
    author='Jeremy Sattefield',
    author_email='jsatt@jsatt.com',
    url='https://github.com/jsatt/django-variant',
    packages=[
        'variant', 'variant.migrations'],
    install_requires=[
        'Django>=1.5', 'six'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Site Management'],
)
